unit RaumschiffThreads;

interface

uses
  Classes, Graphics, SysUtils;

type tplanet=
       record
        x,y:double;
        masse:double;
        radius:integer;
        bitmapnr:integer;
       end;

const maxplaneten=12;
const maxtruemmerstuecke=150;

var planet:array[1..maxplaneten] of tplanet;
var anzplaneten:integer;
var destinationplanet:integer=0;
var anzSchiffeImOrbit:integer=0;
var selbstzerstoerung:boolean=false;
const SpielerRadius=5;

type
  TRaumschiff = class(TThread)
  private
    { Private-Deklarationen }
    Fv0,Fx0,Fy0,Fphi0:double;
    x,y:double;
    xalt,yalt,xuralt,yuralt:integer;
    vx,vy:double;
    aktiv:boolean;
    FSpielerNr,FFarbe:Integer;
    LastTime:TDateTime;
  protected
    procedure Execute; override;
    procedure FliegeEinStueck;
  public
    constructor create(v0,x0,y0,phi0:double;SpielerNr,Farbe:integer);
  end;

type trtruemmer= record
    x,y:double;
    xalt,yalt:integer;
    vx,vy:double;
    aktiv:boolean;
    lastpixel:integer;
    end;

type
  TTruemmer = class(TThread)
  private
    { Private-Deklarationen }
    truemmerfeld:array[1..maxtruemmerstuecke] of trtruemmer;
    Fx0,Fy0:double;
    irgendeintruemmeraktiv:boolean;
    FFarbe:Integer;
    LastTime,
    NowTime:TDateTime;
  protected
    procedure Execute; override;
    procedure FliegeEinStueckt;
  public
    constructor create(x0,y0:double;Farbe:integer);
  end;

implementation
uses schwerkraft,spielernameneingabe;

function fdt(vorher,jetzt:double):Double;
begin
  fdt:=(jetzt-vorher)*24*60*60*20;
end;

function dreieckshoehe(xp,yp,x1,y1,x2,y2:double):double;
{Abstand des Punktes xp,yp von der Gerade durch
 x1,y1 und x2,y1 ber die Formel fr die Dreiecksflche
 A=Grundseite*halbe Hhe, aufgelst nach Hhe)}
begin
  if (x2=x1) and (y2=y1)
  then
  dreieckshoehe:=sqrt(sqr(xp-x1)+sqr(yp-y1))
  else
  dreieckshoehe:=abs(x1*(y2-yp)+x2*(yp-y1)+xp*(y1-y2))
                 /sqrt(sqr(x2-x1)+sqr(y2-y1));
end;

function signum(x:double):integer;
begin
  if x=0 then signum:=0
  else if x<0 then signum:=-1
  else signum:=1
end;

function zsqr(x:double):double;
begin
  zsqr:=sqr(abs(x))*signum(x);
end;

function spielergetroffen(sx,sy,x,y,xa,ya:double):boolean;
{Bei Kollisionsverdacht einfach einmal in Zehntelspunkten nachgucken...}
var d2,dx,dy:double;
var i:integer;
begin
  spielergetroffen:=false;
  d2:=sqr(sx-x)+sqr(sy-y);
  if d2<=sqr(SpielerRadius)+9+sqr(x-xa)+sqr(y-ya)
  then
    begin
      dx:=(x-xa)/10;
      dy:=(y-ya)/10;
      for i:=0 to 9 do
      begin
        d2:=sqr(sx-x-i*dx)+sqr(sy-y-i*dy);
        if d2<=sqr(SpielerRadius)+9
        then
          begin
            spielergetroffen:=true;
            break;
          end;
      end;
    end;
end;

procedure TRaumschiff.FliegeEinStueck;
var j:integer;
var dx,dy,abstand2,abstand3:double;
var s:string;
begin
  with f_universum do begin
  canvas.pen.mode:=pmcopy;
  for j:=1 to anzplaneten do
  begin
    dx:=planet[j].x-x;
    dy:=planet[j].y-y;
    abstand2:=sqr(dx)+sqr(dy);
    abstand3:=abstand2*sqrt(abstand2);
    vx:=vx+dx*planet[j].masse/abstand3;
    vy:=vy+dy*planet[j].masse/abstand3;
    if abstand2>1e9 then aktiv:=false; {Aus dem Planetensystem entkommen}
    if abstand2<=sqr(planet[j].radius)+9
    then {Auf Planeten gestuerzt}
      begin
        destinationplanet:=j;
        aktiv:=false;
        canvas.moveto(xalt,yalt);
        canvas.pen.width:=5;
        canvas.pen.color:=clblack;
        canvas.lineto(round(x),round(y));
      end;
  end;

  for j:=1 to maxSpielerZahl do if spieler[j].aktiv then
  begin
    if Spielergetroffen(spieler[j].x,spieler[j].y,x,y,xalt,yalt)
    then {Spieler getroffen}
      begin
        if j<>FSpielerNr then
        begin
          s:=' (20 Punkte fr';
          inc(Spieler[FSpielerNr].aktpunkte,20);
        end
        else
        begin
          s:=' (17 Strafpunkte fr';
          dec(Spieler[FSpielerNr].aktpunkte,17);
        end;
        slprint(1,Spieler[j].name
                  +' wurde von '
                  +Spieler[FSpielerNr].name
                  +' getroffen.'+s+' '+Spieler[FSpielerNr].name+')');
        Spieler[j].getroffenvon:=FSpielerNr;

        aktiv:=false;
        canvas.moveto(xalt,yalt);
        canvas.pen.width:=3;
        canvas.pen.color:=clblack;
        canvas.lineto(round(x),round(y));
        loeschespieler(j);
      end;
  end;
  if aktiv then
  begin
    xuralt:=xalt;
    yuralt:=yalt;
    xalt:=round(x);
    yalt:=round(y);
    x:=x+vx;
    y:=y+vy;

    canvas.moveto(xuralt,yuralt);
    canvas.pen.width:=3;
    canvas.pen.color:=clblack;
    canvas.lineto(xalt,yalt);
    canvas.pen.color:=FFarbe;
    canvas.lineto(round(x),round(y));
    canvas.pen.color:=$4f4f4f;
    canvas.pen.width:=1;
    canvas.moveto(xuralt,yuralt);
    canvas.lineto(xalt,yalt);
  end;
end; end;

procedure TRaumschiff.Execute;
begin
  x:=fx0;
  y:=fy0;
  xalt:=round(x);
  yalt:=round(y);
  xuralt:=xalt;
  yuralt:=yalt;
  vx:=fv0*cos(fphi0*pi/180);
  vy:=-fv0*sin(fphi0*pi/180);
  inc(anzschiffeimorbit);
  f_universum.orbitinfo;
  LastTime:=Now;
  repeat
    synchronize(fliegeeinstueck);
    if selbstzerstoerung then aktiv:=false;
  until not aktiv;
  dec(anzschiffeimorbit);
  f_universum.orbitinfo;
end;

constructor TRaumschiff.create(v0,x0,y0,phi0:double;SpielerNr,Farbe:integer);
begin
  inherited create(false);
  Fv0:=v0;
  Fx0:=x0;
  Fy0:=y0;
  Fphi0:=phi0;
  FSpielerNr:=SpielerNr;
  FFarbe:=Farbe;
  aktiv:=true;
  FreeOnTerminate:=true;
end;

procedure TTruemmer.FliegeEinStueckt;
var i,j:integer;
var dx,dy,dt,abstand2,abstand3:double;

begin
nowtime:=now;
dt:=fdt(lasttime,nowtime);
lasttime:=nowtime;
irgendeintruemmeraktiv:=false;
with f_universum do
begin
  for i:=1 to maxtruemmerstuecke do
  with truemmerfeld[i] do
  begin
    if aktiv then
    begin
      for j:=1 to anzplaneten do
      begin
        dx:=planet[j].x-x;
        dy:=planet[j].y-y;
        abstand2:=sqr(dx)+sqr(dy);
        abstand3:=abstand2*sqrt(abstand2);
        vx:=vx+dx*planet[j].masse/abstand3*dt;
        vy:=vy+dy*planet[j].masse/abstand3*dt;
        if abstand2>1e8 then aktiv:=false; {Aus dem Planetensystem entkommen}
        if abstand2<=sqr(planet[j].radius)+9
        then {Auf Planeten gestuerzt}
            aktiv:=false;
      end;
    end;
    if aktiv then
    begin
      irgendeintruemmeraktiv:=true;
      xalt:=round(x);
      yalt:=round(y);
      x:=x+vx;
      y:=y+vy;
      if lastpixel=FFarbe
      then canvas.pixels[xalt,yalt]:=0
      else canvas.pixels[xalt,yalt]:=lastpixel;
      lastpixel:=canvas.pixels[round(x),round(y)];
    end;
  end;
  for i:=maxtruemmerstuecke downto 1 do
  with truemmerfeld[i] do
  if aktiv then
  begin
    canvas.pixels[round(x),round(y)]:=FFarbe
  end;
end;end;

procedure TTruemmer.Execute;
var i:integer;
var r,phi:double;
begin
nowtime:=now;
lasttime:=nowtime;
for i:=1 to maxtruemmerstuecke do
with truemmerfeld[i] do
begin
  xalt:=round(x);
  yalt:=round(y);
  phi:=random*2*pi;
  x:=fx0+SpielerRadius*sin(phi);
  y:=fy0+SpielerRadius*cos(phi);
  r:=zsqr(random*0.75);
  vx:=sin(phi)*r;
  vy:=cos(phi)*r;
  aktiv:=true;
end;
  repeat
    synchronize(fliegeeinstueckt);
    if selbstzerstoerung then irgendeintruemmeraktiv:=false;
  until not irgendeintruemmeraktiv;
end;

constructor TTruemmer.create(x0,y0:double;Farbe:integer);
begin
  inherited create(false);
  Fx0:=x0;
  Fy0:=y0;
  FFarbe:=Farbe;
  FreeOnTerminate:=true;
end;

end.


